==> local-booter-23.1.0-331.1639-sources.jar: com/xebialabs/xlplatform/synthetic/xml/XmlOperations.java <==
package com.xebialabs.xlplatform.synthetic.xml;

import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Optional;
import java.util.function.Predicate;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.xebialabs.deployit.plugin.api.reflect.IDescriptorRegistry;
import com.xebialabs.deployit.plugin.api.reflect.Type;

interface XmlOperations {

    IDescriptorRegistry descriptorRegistry();

    default String getRequiredStringAttribute(Element element, String attributeName) {
        if (element.hasAttribute(attributeName)) {
            return element.getAttribute(attributeName);
        }
        throw new IllegalArgumentException("Attribute " + attributeName + " not provided");
    }

    default Type getRequiredTypeAttribute(Element element, String attributeName) {
        Type type = getOptionalTypeAttribute(element, attributeName);
        if (type == null) {
            throw new IllegalArgumentException("Attribute " + attributeName + " not provided");
        }
        return type;
    }

    default Type getOptionalTypeAttribute(Element element, String attributeName, Type defaultValue) {
        String typeAttr = getOptionalStringAttribute(element, attributeName, null);
        Type type = defaultValue;
        if (typeAttr != null) {
            type = descriptorRegistry().lookupType(typeAttr);
        }
        return type;
    }

    default Type getOptionalTypeAttribute(Element element, String attributeName) {
        return getOptionalTypeAttribute(element, attributeName, null);
    }

    default Optional<Type> getOptionalType(Element element, String attributeName) {
        return getOptionalString(element, attributeName).map(descriptorRegistry()::lookupType);
    }

    default boolean getOptionalBooleanAttribute(Element element, String attributeName, boolean defaultValue) {
        return getOptionalBoolean(element, attributeName).orElse(defaultValue);
    }

    default Optional<Boolean> getOptionalBoolean(Element element, String attributeName) {
        return getOptionalString(element, attributeName).map(Boolean::valueOf);
    }

    default String getOptionalStringAttribute(Element element, String attributeName, String defaultValue) {
        return getOptionalString(element, attributeName).orElse(defaultValue);
    }

    default Optional<String> getOptionalString(Element element, String attributeName) {
        if (element.hasAttribute(attributeName)) {
            return Optional.ofNullable(element.getAttribute(attributeName));
        } else {
            return Optional.empty();
        }
    }

    default Optional<String> getOptionalTextOfChild(Element element, String childElementName) {
        Iterator<Element> children = childrenByName(element, childElementName::equals).iterator();
        if (children.hasNext()) {
            return Optional.ofNullable(children.next().getFirstChild().getTextContent());
        }
        return Optional.empty();
    }

    default Iterable<Element> childrenByName(final Element element, final Predicate<String> matcher) {
        return () -> childByName(element, matcher);
    }

    default Iterator<Element> childByName(final Element element, final Predicate<String> matcher) {
        return new Iterator<Element>() {
            private int i = -1;
            private NodeList childNodes = element.getChildNodes();
            private int nextIndex = -1;

            @Override
            public boolean hasNext() {
                if (nextIndex == i) nextIndex = findNext();
                return nextIndex > i;
            }

            @Override
            public Element next() {
                if (nextIndex == i) nextIndex = findNext();
                if (nextIndex < i) throw new NoSuchElementException("There are no more matching elements");
                i = nextIndex;

                return (Element) childNodes.item(i);
            }

            private int findNext() {
                int next = i;
                while (next < childNodes.getLength()) {
                    next++;
                    if (childNodes.item(next) instanceof Element) {
                        Element e = (Element) childNodes.item(next);
                        if (matcher.test(e.getNodeName())) {
                            return next;
                        }
                    }
                }
                return i - 1;
            }

            @Override
            public void remove() {
                throw new UnsupportedOperationException();
            }
        };
    }

    default <E> void forEach(Iterator<E> iterator, Closure<E> closure) {
        while (iterator.hasNext()) {
            closure.call(iterator.next());
        }
    }

}

==> local-booter-23.3.0-509.1300-sources.jar: com/xebialabs/xlplatform/synthetic/xml/XmlOperations.java <==
package com.xebialabs.xlplatform.synthetic.xml;

import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Optional;
import java.util.function.Predicate;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import com.xebialabs.deployit.plugin.api.reflect.IDescriptorRegistry;
import com.xebialabs.deployit.plugin.api.reflect.Type;

interface XmlOperations {

    IDescriptorRegistry descriptorRegistry();

    default String getRequiredStringAttribute(Element element, String attributeName) {
        if (element.hasAttribute(attributeName)) {
            return element.getAttribute(attributeName);
        }
        throw new IllegalArgumentException("Attribute " + attributeName + " not provided");
    }

    default Type getRequiredTypeAttribute(Element element, String attributeName) {
        Type type = getOptionalTypeAttribute(element, attributeName);
        if (type == null) {
            throw new IllegalArgumentException("Attribute " + attributeName + " not provided");
        }
        return type;
    }

    default Type getOptionalTypeAttribute(Element element, String attributeName, Type defaultValue) {
        String typeAttr = getOptionalStringAttribute(element, attributeName, null);
        Type type = defaultValue;
        if (typeAttr != null) {
            type = descriptorRegistry().lookupType(typeAttr);
        }
        return type;
    }

    default Type getOptionalTypeAttribute(Element element, String attributeName) {
        return getOptionalTypeAttribute(element, attributeName, null);
    }

    default Optional<Type> getOptionalType(Element element, String attributeName) {
        return getOptionalString(element, attributeName).map(descriptorRegistry()::lookupType);
    }

    default boolean getOptionalBooleanAttribute(Element element, String attributeName, boolean defaultValue) {
        return getOptionalBoolean(element, attributeName).orElse(defaultValue);
    }

    default Optional<Boolean> getOptionalBoolean(Element element, String attributeName) {
        return getOptionalString(element, attributeName).map(Boolean::valueOf);
    }

    default String getOptionalStringAttribute(Element element, String attributeName, String defaultValue) {
        return getOptionalString(element, attributeName).orElse(defaultValue);
    }

    default Optional<String> getOptionalString(Element element, String attributeName) {
        if (element.hasAttribute(attributeName)) {
            return Optional.of(element.getAttribute(attributeName));
        } else {
            return Optional.empty();
        }
    }

    default Optional<String> getOptionalTextOfChild(Element element, String childElementName) {
        Iterator<Element> children = childrenByName(element, childElementName::equals).iterator();
        if (children.hasNext()) {
            return Optional.ofNullable(children.next().getFirstChild().getTextContent());
        }
        return Optional.empty();
    }

    default Iterable<Element> childrenByName(final Element element, final Predicate<String> matcher) {
        return () -> childByName(element, matcher);
    }

    default Iterator<Element> childByName(final Element element, final Predicate<String> matcher) {
        return new Iterator<Element>() {
            private int i = -1;
            private final NodeList childNodes = element.getChildNodes();
            private int nextIndex = -1;

            @Override
            public boolean hasNext() {
                if (nextIndex == i) nextIndex = findNext();
                return nextIndex > i;
            }

            @Override
            public Element next() {
                if (nextIndex == i) nextIndex = findNext();
                if (nextIndex < i) throw new NoSuchElementException("There are no more matching elements");
                i = nextIndex;

                return (Element) childNodes.item(i);
            }

            private int findNext() {
                int next = i;
                while (next < childNodes.getLength()) {
                    next++;
                    if (childNodes.item(next) instanceof Element) {
                        Element e = (Element) childNodes.item(next);
                        if (matcher.test(e.getNodeName())) {
                            return next;
                        }
                    }
                }
                return i - 1;
            }

            @Override
            public void remove() {
                throw new UnsupportedOperationException();
            }
        };
    }

    default <E> void forEach(Iterator<E> iterator, Closure<E> closure) {
        while (iterator.hasNext()) {
            closure.call(iterator.next());
        }
    }

}
